import { KeyMod, KeyCode } from 'vscode/vscode/vs/base/common/keyCodes';
import { FileAccess } from 'vscode/vscode/vs/base/common/network';
import { isWeb, isMacintosh } from 'vscode/vscode/vs/base/common/platform';
import './media/debug.contribution.css.js';
import './media/debugHover.css.js';
import { registerEditorContribution, EditorContributionInstantiation } from 'vscode/vscode/vs/editor/browser/editorExtensions';
import { localizeWithPath, localize2WithPath } from 'vscode/vscode/vs/nls';
import { MenuRegistry, MenuId } from 'vscode/vscode/vs/platform/actions/common/actions';
import { Extensions as Extensions$3, ConfigurationScope } from 'vscode/vscode/vs/platform/configuration/common/configurationRegistry';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { SyncDescriptor } from 'vscode/vscode/vs/platform/instantiation/common/descriptors';
import 'vscode/vscode/vs/platform/instantiation/common/extensions';
import { Extensions as Extensions$1 } from 'vscode/vscode/vs/platform/quickinput/common/quickAccess';
import { Registry } from 'vscode/vscode/vs/platform/registry/common/platform';
import { EditorPaneDescriptor } from 'vscode/vscode/vs/workbench/browser/editor';
import { ViewPaneContainer } from 'vscode/vscode/vs/workbench/browser/parts/views/viewPaneContainer';
import { Extensions } from 'vscode/vscode/vs/workbench/common/contributions';
import { EditorExtensions } from 'vscode/vscode/vs/workbench/common/editor';
import { Extensions as Extensions$2, ViewContainerLocation } from 'vscode/vscode/vs/workbench/common/views';
import { BreakpointEditorContribution } from 'vscode/vscode/vs/workbench/contrib/debug/browser/breakpointEditorContribution';
import { BreakpointsView } from 'vscode/vscode/vs/workbench/contrib/debug/browser/breakpointsView';
import { CallStackEditorContribution } from 'vscode/vscode/vs/workbench/contrib/debug/browser/callStackEditorContribution';
import { CallStackView } from './callStackView.js';
import { registerColors } from 'vscode/vscode/vs/workbench/contrib/debug/browser/debugColors';
import { DEBUG_QUICK_ACCESS_PREFIX, SELECT_AND_START_ID, DEBUG_CONSOLE_QUICK_ACCESS_PREFIX, SELECT_DEBUG_CONSOLE_ID, DEBUG_COMMAND_CATEGORY, RESTART_SESSION_ID, RESTART_LABEL, TERMINATE_THREAD_ID, STEP_OVER_ID, STEP_OVER_LABEL, STEP_INTO_ID, STEP_INTO_LABEL, STEP_INTO_TARGET_ID, STEP_INTO_TARGET_LABEL, STEP_OUT_ID, STEP_OUT_LABEL, PAUSE_ID, PAUSE_LABEL, DISCONNECT_ID, DISCONNECT_LABEL, DISCONNECT_AND_SUSPEND_ID, DISCONNECT_AND_SUSPEND_LABEL, STOP_ID, STOP_LABEL, CONTINUE_ID, CONTINUE_LABEL, FOCUS_REPL_ID, JUMP_TO_CURSOR_ID, TOGGLE_INLINE_BREAKPOINT_ID, DEBUG_START_COMMAND_ID, DEBUG_START_LABEL, DEBUG_RUN_COMMAND_ID, DEBUG_RUN_LABEL, SELECT_AND_START_LABEL, NEXT_DEBUG_CONSOLE_ID, NEXT_DEBUG_CONSOLE_LABEL, PREV_DEBUG_CONSOLE_ID, PREV_DEBUG_CONSOLE_LABEL, SHOW_LOADED_SCRIPTS_ID, OPEN_LOADED_SCRIPTS_LABEL, SELECT_DEBUG_CONSOLE_LABEL, SELECT_DEBUG_SESSION_ID, SELECT_DEBUG_SESSION_LABEL, CALLSTACK_TOP_ID, CALLSTACK_TOP_LABEL, CALLSTACK_BOTTOM_ID, CALLSTACK_BOTTOM_LABEL, CALLSTACK_UP_ID, CALLSTACK_UP_LABEL, CALLSTACK_DOWN_ID, CALLSTACK_DOWN_LABEL, RESTART_FRAME_ID, COPY_STACK_TRACE_ID, COPY_VALUE_ID, COPY_VALUE_LABEL, COPY_EVALUATE_PATH_ID, COPY_EVALUATE_PATH_LABEL, ADD_TO_WATCH_ID, ADD_TO_WATCH_LABEL, EDIT_EXPRESSION_COMMAND_ID, SET_EXPRESSION_COMMAND_ID, REMOVE_EXPRESSION_COMMAND_ID, ADD_CONFIGURATION_ID } from 'vscode/vscode/vs/workbench/contrib/debug/browser/debugCommands';
import { DebugConsoleQuickAccess } from './debugConsoleQuickAccess.js';
import { RunToCursorAction, SelectionToReplAction, SelectionToWatchExpressionsAction } from './debugEditorActions.js';
import { DebugEditorContribution } from './debugEditorContribution.js';
import { debugInspectMemory, watchExpressionRemove, debugRun, debugConsoleViewIcon, runViewIcon, variablesViewIcon, watchViewIcon, callStackViewIcon, breakpointsViewIcon, loadedScriptsViewIcon } from 'vscode/vscode/vs/workbench/contrib/debug/browser/debugIcons';
import { DebugProgressContribution } from './debugProgress.js';
import { StartDebugQuickAccessProvider } from './debugQuickAccess.js';
import './debugService.js';
import { DebugStatusContribution } from './debugStatus.js';
import { DebugTitleContribution } from './debugTitle.js';
import { DebugToolBar } from './debugToolBar.js';
import { DebugViewPaneContainer } from './debugViewlet.js';
import { DisassemblyViewContribution, DisassemblyView } from './disassemblyView.js';
import { LoadedScriptsView } from './loadedScriptsView.js';
import { Repl } from './repl.js';
import { StatusBarColorProvider } from './statusbarColorProvider.js';
import { VIEW_MEMORY_ID, SET_VARIABLE_ID, BREAK_WHEN_VALUE_IS_READ_ID, BREAK_WHEN_VALUE_CHANGES_ID, BREAK_WHEN_VALUE_IS_ACCESSED_ID, VariablesView } from './variablesView.js';
import { ADD_WATCH_ID, ADD_WATCH_LABEL, REMOVE_WATCH_EXPRESSIONS_COMMAND_ID, REMOVE_WATCH_EXPRESSIONS_LABEL, WatchExpressionsView } from './watchExpressionsView.js';
import { WelcomeView } from './welcomeView.js';
import { BREAKPOINT_EDITOR_CONTRIBUTION_ID, EDITOR_CONTRIBUTION_ID, CONTEXT_DEBUGGERS_AVAILABLE, CONTEXT_IN_DEBUG_MODE, CONTEXT_DEBUG_STATE, CONTEXT_STEP_INTO_TARGETS_SUPPORTED, CONTEXT_FOCUSED_SESSION_IS_NO_DEBUG, CONTEXT_FOCUSED_SESSION_IS_ATTACH, CONTEXT_TERMINATE_DEBUGGEE_SUPPORTED, CONTEXT_SUSPEND_DEBUGGEE_SUPPORTED, CONTEXT_JUMP_TO_CURSOR_SUPPORTED, getStateLabel, State, CONTEXT_CALLSTACK_ITEM_TYPE, CONTEXT_RESTART_FRAME_SUPPORTED, CONTEXT_STACK_FRAME_SUPPORTS_RESTART, CONTEXT_CAN_VIEW_MEMORY, CONTEXT_SET_VARIABLE_SUPPORTED, CONTEXT_VARIABLE_EVALUATE_NAME_PRESENT, CONTEXT_SET_EXPRESSION_SUPPORTED, CONTEXT_VARIABLE_IS_READONLY, CONTEXT_BREAK_WHEN_VALUE_IS_READ_SUPPORTED, CONTEXT_BREAK_WHEN_VALUE_CHANGES_SUPPORTED, CONTEXT_BREAK_WHEN_VALUE_IS_ACCESSED_SUPPORTED, CONTEXT_WATCH_ITEM_TYPE, CONTEXT_VARIABLE_VALUE, DEBUG_PANEL_ID, REPL_VIEW_ID, VIEWLET_ID, VARIABLES_VIEW_ID, CONTEXT_DEBUG_UX, WATCH_VIEW_ID, CALLSTACK_VIEW_ID, BREAKPOINTS_VIEW_ID, CONTEXT_BREAKPOINTS_EXIST, CONTEXT_HAS_DEBUGGED, LOADED_SCRIPTS_VIEW_ID, CONTEXT_LOADED_SCRIPTS_SUPPORTED, DISASSEMBLY_VIEW_ID, INTERNAL_CONSOLE_OPTIONS_SCHEMA } from 'vscode/vscode/vs/workbench/contrib/debug/common/debug';
import 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import { DebugContentProvider } from '../common/debugContentProvider.js';
import { DebugLifecycle } from '../common/debugLifecycle.js';
import '../common/debugVisualizers.js';
import { DisassemblyViewInput } from 'vscode/vscode/vs/workbench/contrib/debug/common/disassemblyViewInput';
import { COPY_NOTEBOOK_VARIABLE_VALUE_ID, COPY_NOTEBOOK_VARIABLE_VALUE_LABEL } from '../../notebook/browser/contrib/notebookVariables/notebookVariableCommands.js';
import { launchSchemaId } from 'vscode/vscode/vs/workbench/services/configuration/common/configuration';
import { LifecyclePhase } from 'vscode/vscode/vs/workbench/services/lifecycle/common/lifecycle';

const _moduleId = "vs/workbench/contrib/debug/browser/debug.contribution";
const debugCategory = ( localizeWithPath(_moduleId, 0, "Debug"));
registerColors();
( (Registry.as(Extensions.Workbench))).registerWorkbenchContribution(DebugStatusContribution, LifecyclePhase.Eventually);
( (Registry.as(Extensions.Workbench))).registerWorkbenchContribution(DebugProgressContribution, LifecyclePhase.Eventually);
if (isWeb) {
    ( (Registry.as(Extensions.Workbench))).registerWorkbenchContribution(DebugTitleContribution, LifecyclePhase.Eventually);
}
( (Registry.as(Extensions.Workbench))).registerWorkbenchContribution(DebugToolBar, LifecyclePhase.Restored);
( (Registry.as(Extensions.Workbench))).registerWorkbenchContribution(DebugContentProvider, LifecyclePhase.Eventually);
( (Registry.as(Extensions.Workbench))).registerWorkbenchContribution(StatusBarColorProvider, LifecyclePhase.Eventually);
( (Registry.as(Extensions.Workbench))).registerWorkbenchContribution(DisassemblyViewContribution, LifecyclePhase.Eventually);
( (Registry.as(Extensions.Workbench))).registerWorkbenchContribution(DebugLifecycle, LifecyclePhase.Eventually);
( (Registry.as(Extensions$1.Quickaccess))).registerQuickAccessProvider({
    ctor: StartDebugQuickAccessProvider,
    prefix: DEBUG_QUICK_ACCESS_PREFIX,
    contextKey: 'inLaunchConfigurationsPicker',
    placeholder: ( localizeWithPath(_moduleId, 1, "Type the name of a launch configuration to run.")),
    helpEntries: [{
            description: ( localizeWithPath(_moduleId, 2, "Start Debugging")),
            commandId: SELECT_AND_START_ID,
            commandCenterOrder: 50
        }]
});
( (Registry.as(Extensions$1.Quickaccess))).registerQuickAccessProvider({
    ctor: DebugConsoleQuickAccess,
    prefix: DEBUG_CONSOLE_QUICK_ACCESS_PREFIX,
    contextKey: 'inDebugConsolePicker',
    placeholder: ( localizeWithPath(_moduleId, 3, "Type the name of a debug console to open.")),
    helpEntries: [{ description: ( localizeWithPath(_moduleId, 4, "Show All Debug Consoles")), commandId: SELECT_DEBUG_CONSOLE_ID }]
});
registerEditorContribution('editor.contrib.callStack', CallStackEditorContribution, EditorContributionInstantiation.AfterFirstRender);
registerEditorContribution(BREAKPOINT_EDITOR_CONTRIBUTION_ID, BreakpointEditorContribution, EditorContributionInstantiation.AfterFirstRender);
registerEditorContribution(EDITOR_CONTRIBUTION_ID, DebugEditorContribution, EditorContributionInstantiation.BeforeFirstInteraction);
const registerDebugCommandPaletteItem = (id, title, when, precondition) => {
    MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
        when: ( (ContextKeyExpr.and(CONTEXT_DEBUGGERS_AVAILABLE, when))),
        group: debugCategory,
        command: {
            id,
            title,
            category: DEBUG_COMMAND_CATEGORY,
            precondition
        }
    });
};
registerDebugCommandPaletteItem(RESTART_SESSION_ID, RESTART_LABEL);
registerDebugCommandPaletteItem(TERMINATE_THREAD_ID, ( localize2WithPath(_moduleId, 5, "Terminate Thread")), CONTEXT_IN_DEBUG_MODE);
registerDebugCommandPaletteItem(STEP_OVER_ID, STEP_OVER_LABEL, CONTEXT_IN_DEBUG_MODE, ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))));
registerDebugCommandPaletteItem(STEP_INTO_ID, STEP_INTO_LABEL, CONTEXT_IN_DEBUG_MODE, ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))));
registerDebugCommandPaletteItem(STEP_INTO_TARGET_ID, STEP_INTO_TARGET_LABEL, CONTEXT_IN_DEBUG_MODE, ( (ContextKeyExpr.and(
    CONTEXT_STEP_INTO_TARGETS_SUPPORTED,
    CONTEXT_IN_DEBUG_MODE,
     (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))
))));
registerDebugCommandPaletteItem(STEP_OUT_ID, STEP_OUT_LABEL, CONTEXT_IN_DEBUG_MODE, ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))));
registerDebugCommandPaletteItem(PAUSE_ID, PAUSE_LABEL, CONTEXT_IN_DEBUG_MODE, ( (ContextKeyExpr.and(
     (CONTEXT_DEBUG_STATE.isEqualTo('running')),
     (CONTEXT_FOCUSED_SESSION_IS_NO_DEBUG.toNegated())
))));
registerDebugCommandPaletteItem(DISCONNECT_ID, DISCONNECT_LABEL, CONTEXT_IN_DEBUG_MODE, ( (ContextKeyExpr.or(CONTEXT_FOCUSED_SESSION_IS_ATTACH, CONTEXT_TERMINATE_DEBUGGEE_SUPPORTED))));
registerDebugCommandPaletteItem(DISCONNECT_AND_SUSPEND_ID, DISCONNECT_AND_SUSPEND_LABEL, CONTEXT_IN_DEBUG_MODE, ( (ContextKeyExpr.or(
    CONTEXT_FOCUSED_SESSION_IS_ATTACH,
     (ContextKeyExpr.and(CONTEXT_SUSPEND_DEBUGGEE_SUPPORTED, CONTEXT_TERMINATE_DEBUGGEE_SUPPORTED))
))));
registerDebugCommandPaletteItem(STOP_ID, STOP_LABEL, CONTEXT_IN_DEBUG_MODE, ( (ContextKeyExpr.or(
     (CONTEXT_FOCUSED_SESSION_IS_ATTACH.toNegated()),
    CONTEXT_TERMINATE_DEBUGGEE_SUPPORTED
))));
registerDebugCommandPaletteItem(CONTINUE_ID, CONTINUE_LABEL, CONTEXT_IN_DEBUG_MODE, ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))));
registerDebugCommandPaletteItem(FOCUS_REPL_ID, ( localize2WithPath(_moduleId, 6, "Focus on Debug Console View")));
registerDebugCommandPaletteItem(JUMP_TO_CURSOR_ID, ( localize2WithPath(_moduleId, 7, "Jump to Cursor")), CONTEXT_JUMP_TO_CURSOR_SUPPORTED);
registerDebugCommandPaletteItem(JUMP_TO_CURSOR_ID, ( localize2WithPath(_moduleId, 8, "Set Next Statement")), CONTEXT_JUMP_TO_CURSOR_SUPPORTED);
registerDebugCommandPaletteItem(RunToCursorAction.ID, RunToCursorAction.LABEL, CONTEXT_DEBUGGERS_AVAILABLE);
registerDebugCommandPaletteItem(SelectionToReplAction.ID, SelectionToReplAction.LABEL, CONTEXT_IN_DEBUG_MODE);
registerDebugCommandPaletteItem(SelectionToWatchExpressionsAction.ID, SelectionToWatchExpressionsAction.LABEL);
registerDebugCommandPaletteItem(TOGGLE_INLINE_BREAKPOINT_ID, ( localize2WithPath(_moduleId, 9, "Inline Breakpoint")));
registerDebugCommandPaletteItem(DEBUG_START_COMMAND_ID, DEBUG_START_LABEL, ( (ContextKeyExpr.and(
    CONTEXT_DEBUGGERS_AVAILABLE,
     (CONTEXT_DEBUG_STATE.notEqualsTo(getStateLabel(State.Initializing)))
))));
registerDebugCommandPaletteItem(DEBUG_RUN_COMMAND_ID, DEBUG_RUN_LABEL, ( (ContextKeyExpr.and(
    CONTEXT_DEBUGGERS_AVAILABLE,
     (CONTEXT_DEBUG_STATE.notEqualsTo(getStateLabel(State.Initializing)))
))));
registerDebugCommandPaletteItem(SELECT_AND_START_ID, SELECT_AND_START_LABEL, ( (ContextKeyExpr.and(
    CONTEXT_DEBUGGERS_AVAILABLE,
     (CONTEXT_DEBUG_STATE.notEqualsTo(getStateLabel(State.Initializing)))
))));
registerDebugCommandPaletteItem(NEXT_DEBUG_CONSOLE_ID, NEXT_DEBUG_CONSOLE_LABEL);
registerDebugCommandPaletteItem(PREV_DEBUG_CONSOLE_ID, PREV_DEBUG_CONSOLE_LABEL);
registerDebugCommandPaletteItem(SHOW_LOADED_SCRIPTS_ID, OPEN_LOADED_SCRIPTS_LABEL, CONTEXT_IN_DEBUG_MODE);
registerDebugCommandPaletteItem(SELECT_DEBUG_CONSOLE_ID, SELECT_DEBUG_CONSOLE_LABEL);
registerDebugCommandPaletteItem(SELECT_DEBUG_SESSION_ID, SELECT_DEBUG_SESSION_LABEL);
registerDebugCommandPaletteItem(CALLSTACK_TOP_ID, CALLSTACK_TOP_LABEL, CONTEXT_IN_DEBUG_MODE, ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))));
registerDebugCommandPaletteItem(CALLSTACK_BOTTOM_ID, CALLSTACK_BOTTOM_LABEL, CONTEXT_IN_DEBUG_MODE, ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))));
registerDebugCommandPaletteItem(CALLSTACK_UP_ID, CALLSTACK_UP_LABEL, CONTEXT_IN_DEBUG_MODE, ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))));
registerDebugCommandPaletteItem(CALLSTACK_DOWN_ID, CALLSTACK_DOWN_LABEL, CONTEXT_IN_DEBUG_MODE, ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))));
const registerDebugViewMenuItem = (menuId, id, title, order, when, precondition, group = 'navigation', icon) => {
    MenuRegistry.appendMenuItem(menuId, {
        group,
        when,
        order,
        icon,
        command: {
            id,
            title,
            icon,
            precondition
        }
    });
};
registerDebugViewMenuItem(MenuId.DebugCallStackContext, RESTART_SESSION_ID, RESTART_LABEL, 10, ( (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('session'))), undefined, '3_modification');
registerDebugViewMenuItem(MenuId.DebugCallStackContext, DISCONNECT_ID, DISCONNECT_LABEL, 20, ( (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('session'))), undefined, '3_modification');
registerDebugViewMenuItem(MenuId.DebugCallStackContext, DISCONNECT_AND_SUSPEND_ID, DISCONNECT_AND_SUSPEND_LABEL, 21, ( (ContextKeyExpr.and(
     (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('session')),
    CONTEXT_SUSPEND_DEBUGGEE_SUPPORTED,
    CONTEXT_TERMINATE_DEBUGGEE_SUPPORTED
))), undefined, '3_modification');
registerDebugViewMenuItem(MenuId.DebugCallStackContext, STOP_ID, STOP_LABEL, 30, ( (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('session'))), undefined, '3_modification');
registerDebugViewMenuItem(MenuId.DebugCallStackContext, PAUSE_ID, PAUSE_LABEL, 10, ( (ContextKeyExpr.and(
     (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('thread')),
     (ContextKeyExpr.and(
         (CONTEXT_DEBUG_STATE.isEqualTo('running')),
         (CONTEXT_FOCUSED_SESSION_IS_NO_DEBUG.toNegated())
    ))
))));
registerDebugViewMenuItem(MenuId.DebugCallStackContext, CONTINUE_ID, CONTINUE_LABEL, 10, ( (ContextKeyExpr.and(
     (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('thread')),
     (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))
))));
registerDebugViewMenuItem(MenuId.DebugCallStackContext, STEP_OVER_ID, STEP_OVER_LABEL, 20, ( (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('thread'))), ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))));
registerDebugViewMenuItem(MenuId.DebugCallStackContext, STEP_INTO_ID, STEP_INTO_LABEL, 30, ( (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('thread'))), ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))));
registerDebugViewMenuItem(MenuId.DebugCallStackContext, STEP_OUT_ID, STEP_OUT_LABEL, 40, ( (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('thread'))), ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))));
registerDebugViewMenuItem(MenuId.DebugCallStackContext, TERMINATE_THREAD_ID, ( localizeWithPath(_moduleId, 5, "Terminate Thread")), 10, ( (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('thread'))), undefined, 'termination');
registerDebugViewMenuItem(MenuId.DebugCallStackContext, RESTART_FRAME_ID, ( localizeWithPath(_moduleId, 10, "Restart Frame")), 10, ( (ContextKeyExpr.and(
     (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('stackFrame')),
    CONTEXT_RESTART_FRAME_SUPPORTED
))), CONTEXT_STACK_FRAME_SUPPORTS_RESTART);
registerDebugViewMenuItem(MenuId.DebugCallStackContext, COPY_STACK_TRACE_ID, ( localizeWithPath(_moduleId, 11, "Copy Call Stack")), 20, ( (CONTEXT_CALLSTACK_ITEM_TYPE.isEqualTo('stackFrame'))), undefined, '3_modification');
registerDebugViewMenuItem(MenuId.DebugVariablesContext, VIEW_MEMORY_ID, ( localizeWithPath(_moduleId, 12, "View Binary Data")), 15, CONTEXT_CAN_VIEW_MEMORY, CONTEXT_IN_DEBUG_MODE, 'inline', debugInspectMemory);
registerDebugViewMenuItem(MenuId.DebugVariablesContext, SET_VARIABLE_ID, ( localizeWithPath(_moduleId, 13, "Set Value")), 10, ( (ContextKeyExpr.or(
    CONTEXT_SET_VARIABLE_SUPPORTED,
     (ContextKeyExpr.and(CONTEXT_VARIABLE_EVALUATE_NAME_PRESENT, CONTEXT_SET_EXPRESSION_SUPPORTED))
))), ( (CONTEXT_VARIABLE_IS_READONLY.toNegated())), '3_modification');
registerDebugViewMenuItem(MenuId.DebugVariablesContext, COPY_VALUE_ID, COPY_VALUE_LABEL, 10, undefined, undefined, '5_cutcopypaste');
registerDebugViewMenuItem(MenuId.DebugVariablesContext, COPY_EVALUATE_PATH_ID, COPY_EVALUATE_PATH_LABEL, 20, CONTEXT_VARIABLE_EVALUATE_NAME_PRESENT, undefined, '5_cutcopypaste');
registerDebugViewMenuItem(MenuId.DebugVariablesContext, ADD_TO_WATCH_ID, ADD_TO_WATCH_LABEL, 100, CONTEXT_VARIABLE_EVALUATE_NAME_PRESENT, undefined, 'z_commands');
registerDebugViewMenuItem(MenuId.DebugVariablesContext, BREAK_WHEN_VALUE_IS_READ_ID, ( localizeWithPath(_moduleId, 14, "Break on Value Read")), 200, CONTEXT_BREAK_WHEN_VALUE_IS_READ_SUPPORTED, undefined, 'z_commands');
registerDebugViewMenuItem(MenuId.DebugVariablesContext, BREAK_WHEN_VALUE_CHANGES_ID, ( localizeWithPath(_moduleId, 15, "Break on Value Change")), 210, CONTEXT_BREAK_WHEN_VALUE_CHANGES_SUPPORTED, undefined, 'z_commands');
registerDebugViewMenuItem(MenuId.DebugVariablesContext, BREAK_WHEN_VALUE_IS_ACCESSED_ID, ( localizeWithPath(_moduleId, 16, "Break on Value Access")), 220, CONTEXT_BREAK_WHEN_VALUE_IS_ACCESSED_SUPPORTED, undefined, 'z_commands');
registerDebugViewMenuItem(MenuId.DebugHoverContext, VIEW_MEMORY_ID, ( localizeWithPath(_moduleId, 12, "View Binary Data")), 15, CONTEXT_CAN_VIEW_MEMORY, CONTEXT_IN_DEBUG_MODE, 'inline', debugInspectMemory);
registerDebugViewMenuItem(MenuId.DebugHoverContext, COPY_VALUE_ID, COPY_VALUE_LABEL, 10, undefined, undefined, '5_cutcopypaste');
registerDebugViewMenuItem(MenuId.DebugHoverContext, COPY_EVALUATE_PATH_ID, COPY_EVALUATE_PATH_LABEL, 20, CONTEXT_VARIABLE_EVALUATE_NAME_PRESENT, undefined, '5_cutcopypaste');
registerDebugViewMenuItem(MenuId.DebugHoverContext, ADD_TO_WATCH_ID, ADD_TO_WATCH_LABEL, 100, CONTEXT_VARIABLE_EVALUATE_NAME_PRESENT, undefined, 'z_commands');
registerDebugViewMenuItem(MenuId.DebugHoverContext, BREAK_WHEN_VALUE_IS_READ_ID, ( localizeWithPath(_moduleId, 14, "Break on Value Read")), 200, CONTEXT_BREAK_WHEN_VALUE_IS_READ_SUPPORTED, undefined, 'z_commands');
registerDebugViewMenuItem(MenuId.DebugHoverContext, BREAK_WHEN_VALUE_CHANGES_ID, ( localizeWithPath(_moduleId, 15, "Break on Value Change")), 210, CONTEXT_BREAK_WHEN_VALUE_CHANGES_SUPPORTED, undefined, 'z_commands');
registerDebugViewMenuItem(MenuId.DebugHoverContext, BREAK_WHEN_VALUE_IS_ACCESSED_ID, ( localizeWithPath(_moduleId, 16, "Break on Value Access")), 220, CONTEXT_BREAK_WHEN_VALUE_IS_ACCESSED_SUPPORTED, undefined, 'z_commands');
registerDebugViewMenuItem(MenuId.DebugWatchContext, ADD_WATCH_ID, ADD_WATCH_LABEL, 10, undefined, undefined, '3_modification');
registerDebugViewMenuItem(MenuId.DebugWatchContext, EDIT_EXPRESSION_COMMAND_ID, ( localizeWithPath(_moduleId, 17, "Edit Expression")), 20, ( (CONTEXT_WATCH_ITEM_TYPE.isEqualTo('expression'))), undefined, '3_modification');
registerDebugViewMenuItem(MenuId.DebugWatchContext, SET_EXPRESSION_COMMAND_ID, ( localizeWithPath(_moduleId, 13, "Set Value")), 30, ( (ContextKeyExpr.or( (ContextKeyExpr.and(
     (CONTEXT_WATCH_ITEM_TYPE.isEqualTo('expression')),
    CONTEXT_SET_EXPRESSION_SUPPORTED
)),  (ContextKeyExpr.and(
     (CONTEXT_WATCH_ITEM_TYPE.isEqualTo('variable')),
    CONTEXT_SET_VARIABLE_SUPPORTED
))))), ( (CONTEXT_VARIABLE_IS_READONLY.toNegated())), '3_modification');
registerDebugViewMenuItem(MenuId.DebugWatchContext, COPY_VALUE_ID, ( localizeWithPath(_moduleId, 18, "Copy Value")), 40, ( (ContextKeyExpr.or(
     (CONTEXT_WATCH_ITEM_TYPE.isEqualTo('expression')),
     (CONTEXT_WATCH_ITEM_TYPE.isEqualTo('variable'))
))), CONTEXT_IN_DEBUG_MODE, '3_modification');
registerDebugViewMenuItem(MenuId.DebugWatchContext, VIEW_MEMORY_ID, ( localizeWithPath(_moduleId, 12, "View Binary Data")), 10, CONTEXT_CAN_VIEW_MEMORY, undefined, 'inline', debugInspectMemory);
registerDebugViewMenuItem(MenuId.DebugWatchContext, REMOVE_EXPRESSION_COMMAND_ID, ( localizeWithPath(_moduleId, 19, "Remove Expression")), 20, ( (CONTEXT_WATCH_ITEM_TYPE.isEqualTo('expression'))), undefined, 'inline', watchExpressionRemove);
registerDebugViewMenuItem(MenuId.DebugWatchContext, REMOVE_WATCH_EXPRESSIONS_COMMAND_ID, REMOVE_WATCH_EXPRESSIONS_LABEL, 20, undefined, undefined, 'z_commands');
registerDebugViewMenuItem(MenuId.NotebookVariablesContext, COPY_NOTEBOOK_VARIABLE_VALUE_ID, COPY_NOTEBOOK_VARIABLE_VALUE_LABEL, 20, CONTEXT_VARIABLE_VALUE);
if (isMacintosh) {
    const registerTouchBarEntry = (id, title, order, when, iconUri) => {
        MenuRegistry.appendMenuItem(MenuId.TouchBarContext, {
            command: {
                id,
                title,
                icon: { dark: iconUri }
            },
            when: ( (ContextKeyExpr.and(CONTEXT_DEBUGGERS_AVAILABLE, when))),
            group: '9_debug',
            order
        });
    };
    registerTouchBarEntry(DEBUG_RUN_COMMAND_ID, DEBUG_RUN_LABEL, 0, ( (CONTEXT_IN_DEBUG_MODE.toNegated())), ( (FileAccess.asFileUri('vs/workbench/contrib/debug/browser/media/continue-tb.png'))));
    registerTouchBarEntry(DEBUG_START_COMMAND_ID, DEBUG_START_LABEL, 1, ( (CONTEXT_IN_DEBUG_MODE.toNegated())), ( (FileAccess.asFileUri('vs/workbench/contrib/debug/browser/media/run-with-debugging-tb.png'))));
    registerTouchBarEntry(CONTINUE_ID, CONTINUE_LABEL, 0, ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped'))), ( (FileAccess.asFileUri('vs/workbench/contrib/debug/browser/media/continue-tb.png'))));
    registerTouchBarEntry(PAUSE_ID, PAUSE_LABEL, 1, ( (ContextKeyExpr.and(CONTEXT_IN_DEBUG_MODE,  (ContextKeyExpr.and(
         (CONTEXT_DEBUG_STATE.isEqualTo('running')),
         (CONTEXT_FOCUSED_SESSION_IS_NO_DEBUG.toNegated())
    ))))), ( (FileAccess.asFileUri('vs/workbench/contrib/debug/browser/media/pause-tb.png'))));
    registerTouchBarEntry(STEP_OVER_ID, STEP_OVER_LABEL, 2, CONTEXT_IN_DEBUG_MODE, ( (FileAccess.asFileUri('vs/workbench/contrib/debug/browser/media/stepover-tb.png'))));
    registerTouchBarEntry(STEP_INTO_ID, STEP_INTO_LABEL, 3, CONTEXT_IN_DEBUG_MODE, ( (FileAccess.asFileUri('vs/workbench/contrib/debug/browser/media/stepinto-tb.png'))));
    registerTouchBarEntry(STEP_OUT_ID, STEP_OUT_LABEL, 4, CONTEXT_IN_DEBUG_MODE, ( (FileAccess.asFileUri('vs/workbench/contrib/debug/browser/media/stepout-tb.png'))));
    registerTouchBarEntry(RESTART_SESSION_ID, RESTART_LABEL, 5, CONTEXT_IN_DEBUG_MODE, ( (FileAccess.asFileUri('vs/workbench/contrib/debug/browser/media/restart-tb.png'))));
    registerTouchBarEntry(STOP_ID, STOP_LABEL, 6, CONTEXT_IN_DEBUG_MODE, ( (FileAccess.asFileUri('vs/workbench/contrib/debug/browser/media/stop-tb.png'))));
}
MenuRegistry.appendMenuItem(MenuId.EditorTitle, { submenu: MenuId.EditorTitleRun, rememberDefaultAction: true, title: ( localize2WithPath(_moduleId, 20, "Run or Debug...")), icon: debugRun, group: 'navigation', order: -1 });
MenuRegistry.appendMenuItem(MenuId.MenubarMainMenu, {
    submenu: MenuId.MenubarDebugMenu,
    title: {
        ...( localize2WithPath(_moduleId, 21, "Run")),
        mnemonicTitle: ( localizeWithPath(_moduleId, 22, "&&Run"))
    },
    order: 6
});
MenuRegistry.appendMenuItem(MenuId.MenubarDebugMenu, {
    group: '1_debug',
    command: {
        id: DEBUG_START_COMMAND_ID,
        title: ( localizeWithPath(_moduleId, 23, "&&Start Debugging"))
    },
    order: 1,
    when: CONTEXT_DEBUGGERS_AVAILABLE
});
MenuRegistry.appendMenuItem(MenuId.MenubarDebugMenu, {
    group: '1_debug',
    command: {
        id: DEBUG_RUN_COMMAND_ID,
        title: ( localizeWithPath(_moduleId, 24, "Run &&Without Debugging"))
    },
    order: 2,
    when: CONTEXT_DEBUGGERS_AVAILABLE
});
MenuRegistry.appendMenuItem(MenuId.MenubarDebugMenu, {
    group: '1_debug',
    command: {
        id: STOP_ID,
        title: ( localizeWithPath(_moduleId, 25, "&&Stop Debugging")),
        precondition: CONTEXT_IN_DEBUG_MODE
    },
    order: 3,
    when: CONTEXT_DEBUGGERS_AVAILABLE
});
MenuRegistry.appendMenuItem(MenuId.MenubarDebugMenu, {
    group: '1_debug',
    command: {
        id: RESTART_SESSION_ID,
        title: ( localizeWithPath(_moduleId, 26, "&&Restart Debugging")),
        precondition: CONTEXT_IN_DEBUG_MODE
    },
    order: 4,
    when: CONTEXT_DEBUGGERS_AVAILABLE
});
MenuRegistry.appendMenuItem(MenuId.MenubarDebugMenu, {
    group: '2_configuration',
    command: {
        id: ADD_CONFIGURATION_ID,
        title: ( localizeWithPath(_moduleId, 27, "A&&dd Configuration..."))
    },
    order: 2,
    when: CONTEXT_DEBUGGERS_AVAILABLE
});
MenuRegistry.appendMenuItem(MenuId.MenubarDebugMenu, {
    group: '3_step',
    command: {
        id: STEP_OVER_ID,
        title: ( localizeWithPath(_moduleId, 28, "Step &&Over")),
        precondition: ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped')))
    },
    order: 1,
    when: CONTEXT_DEBUGGERS_AVAILABLE
});
MenuRegistry.appendMenuItem(MenuId.MenubarDebugMenu, {
    group: '3_step',
    command: {
        id: STEP_INTO_ID,
        title: ( localizeWithPath(_moduleId, 29, "Step &&Into")),
        precondition: ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped')))
    },
    order: 2,
    when: CONTEXT_DEBUGGERS_AVAILABLE
});
MenuRegistry.appendMenuItem(MenuId.MenubarDebugMenu, {
    group: '3_step',
    command: {
        id: STEP_OUT_ID,
        title: ( localizeWithPath(_moduleId, 30, "Step O&&ut")),
        precondition: ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped')))
    },
    order: 3,
    when: CONTEXT_DEBUGGERS_AVAILABLE
});
MenuRegistry.appendMenuItem(MenuId.MenubarDebugMenu, {
    group: '3_step',
    command: {
        id: CONTINUE_ID,
        title: ( localizeWithPath(_moduleId, 31, "&&Continue")),
        precondition: ( (CONTEXT_DEBUG_STATE.isEqualTo('stopped')))
    },
    order: 4,
    when: CONTEXT_DEBUGGERS_AVAILABLE
});
MenuRegistry.appendMenuItem(MenuId.MenubarNewBreakpointMenu, {
    group: '1_breakpoints',
    command: {
        id: TOGGLE_INLINE_BREAKPOINT_ID,
        title: ( localizeWithPath(_moduleId, 32, "Inline Breakp&&oint"))
    },
    order: 2,
    when: CONTEXT_DEBUGGERS_AVAILABLE
});
MenuRegistry.appendMenuItem(MenuId.MenubarDebugMenu, {
    group: '4_new_breakpoint',
    title: ( localizeWithPath(_moduleId, 33, "&&New Breakpoint")),
    submenu: MenuId.MenubarNewBreakpointMenu,
    order: 2,
    when: CONTEXT_DEBUGGERS_AVAILABLE
});
MenuRegistry.appendMenuItem(MenuId.MenubarDebugMenu, {
    group: 'z_install',
    command: {
        id: 'debug.installAdditionalDebuggers',
        title: ( localizeWithPath(_moduleId, 34, "&&Install Additional Debuggers..."))
    },
    order: 1
});
const VIEW_CONTAINER = ( (Registry.as(Extensions$2.ViewContainersRegistry))).registerViewContainer({
    id: DEBUG_PANEL_ID,
    title: ( localize2WithPath(_moduleId, 35, "Debug Console")),
    icon: debugConsoleViewIcon,
    ctorDescriptor: ( (new SyncDescriptor(
        ViewPaneContainer,
        [DEBUG_PANEL_ID, { mergeViewWithContainerWhenSingleView: true }]
    ))),
    storageId: DEBUG_PANEL_ID,
    hideIfEmpty: true,
    order: 2,
}, ViewContainerLocation.Panel, { doNotRegisterOpenCommand: true });
( (Registry.as(Extensions$2.ViewsRegistry))).registerViews([{
        id: REPL_VIEW_ID,
        name: ( localize2WithPath(_moduleId, 35, "Debug Console")),
        containerIcon: debugConsoleViewIcon,
        canToggleVisibility: false,
        canMoveView: true,
        when: CONTEXT_DEBUGGERS_AVAILABLE,
        ctorDescriptor: ( (new SyncDescriptor(Repl))),
        openCommandActionDescriptor: {
            id: 'workbench.debug.action.toggleRepl',
            mnemonicTitle: ( localizeWithPath(_moduleId, 36, "De&&bug Console")),
            keybindings: { primary: KeyMod.CtrlCmd | KeyMod.Shift | KeyCode.KeyY },
            order: 2
        }
    }], VIEW_CONTAINER);
const viewContainer = ( (Registry.as(Extensions$2.ViewContainersRegistry))).registerViewContainer({
    id: VIEWLET_ID,
    title: ( localize2WithPath(_moduleId, 37, "Run and Debug")),
    openCommandActionDescriptor: {
        id: VIEWLET_ID,
        mnemonicTitle: ( localizeWithPath(_moduleId, 38, "&&Run")),
        keybindings: { primary: KeyMod.CtrlCmd | KeyMod.Shift | KeyCode.KeyD },
        order: 3
    },
    ctorDescriptor: ( (new SyncDescriptor(DebugViewPaneContainer))),
    icon: runViewIcon,
    alwaysUseContainerInfo: true,
    order: 3,
}, ViewContainerLocation.Sidebar);
const viewsRegistry = ( (Registry.as(Extensions$2.ViewsRegistry)));
viewsRegistry.registerViews([{ id: VARIABLES_VIEW_ID, name: ( localize2WithPath(_moduleId, 39, "Variables")), containerIcon: variablesViewIcon, ctorDescriptor: ( (new SyncDescriptor(VariablesView))), order: 10, weight: 40, canToggleVisibility: true, canMoveView: true, focusCommand: { id: 'workbench.debug.action.focusVariablesView' }, when: ( (CONTEXT_DEBUG_UX.isEqualTo('default'))) }], viewContainer);
viewsRegistry.registerViews([{ id: WATCH_VIEW_ID, name: ( localize2WithPath(_moduleId, 40, "Watch")), containerIcon: watchViewIcon, ctorDescriptor: ( (new SyncDescriptor(WatchExpressionsView))), order: 20, weight: 10, canToggleVisibility: true, canMoveView: true, focusCommand: { id: 'workbench.debug.action.focusWatchView' }, when: ( (CONTEXT_DEBUG_UX.isEqualTo('default'))) }], viewContainer);
viewsRegistry.registerViews([{ id: CALLSTACK_VIEW_ID, name: ( localize2WithPath(_moduleId, 41, "Call Stack")), containerIcon: callStackViewIcon, ctorDescriptor: ( (new SyncDescriptor(CallStackView))), order: 30, weight: 30, canToggleVisibility: true, canMoveView: true, focusCommand: { id: 'workbench.debug.action.focusCallStackView' }, when: ( (CONTEXT_DEBUG_UX.isEqualTo('default'))) }], viewContainer);
viewsRegistry.registerViews([{ id: BREAKPOINTS_VIEW_ID, name: ( localize2WithPath(_moduleId, 42, "Breakpoints")), containerIcon: breakpointsViewIcon, ctorDescriptor: ( (new SyncDescriptor(BreakpointsView))), order: 40, weight: 20, canToggleVisibility: true, canMoveView: true, focusCommand: { id: 'workbench.debug.action.focusBreakpointsView' }, when: ( (ContextKeyExpr.or(
    CONTEXT_BREAKPOINTS_EXIST,
     (CONTEXT_DEBUG_UX.isEqualTo('default')),
    CONTEXT_HAS_DEBUGGED
))) }], viewContainer);
viewsRegistry.registerViews([{ id: WelcomeView.ID, name: WelcomeView.LABEL, containerIcon: runViewIcon, ctorDescriptor: ( (new SyncDescriptor(WelcomeView))), order: 1, weight: 40, canToggleVisibility: true, when: ( (CONTEXT_DEBUG_UX.isEqualTo('simple'))) }], viewContainer);
viewsRegistry.registerViews([{ id: LOADED_SCRIPTS_VIEW_ID, name: ( localize2WithPath(_moduleId, 43, "Loaded Scripts")), containerIcon: loadedScriptsViewIcon, ctorDescriptor: ( (new SyncDescriptor(LoadedScriptsView))), order: 35, weight: 5, canToggleVisibility: true, canMoveView: true, collapsed: true, when: ( (ContextKeyExpr.and(
    CONTEXT_LOADED_SCRIPTS_SUPPORTED,
     (CONTEXT_DEBUG_UX.isEqualTo('default'))
))) }], viewContainer);
( (Registry.as(EditorExtensions.EditorPane))).registerEditorPane(EditorPaneDescriptor.create(DisassemblyView, DISASSEMBLY_VIEW_ID, ( localizeWithPath(_moduleId, 44, "Disassembly"))), [( (new SyncDescriptor(DisassemblyViewInput)))]);
const configurationRegistry = ( (Registry.as(Extensions$3.Configuration)));
configurationRegistry.registerConfiguration({
    id: 'debug',
    order: 20,
    title: ( localizeWithPath(_moduleId, 45, "Debug")),
    type: 'object',
    properties: {
        'debug.showVariableTypes': {
            type: 'boolean',
            description: ( localizeWithPath(_moduleId, 46, "Show variable type in variable pane during debug session")),
            default: false
        },
        'debug.allowBreakpointsEverywhere': {
            type: 'boolean',
            description: ( localizeWithPath(_moduleId, 47, "Allow setting breakpoints in any file.")),
            default: false
        },
        'debug.gutterMiddleClickAction': {
            type: 'string',
            enum: ['logpoint', 'conditionalBreakpoint', 'triggeredBreakpoint', 'none'],
            description: ( localizeWithPath(
                _moduleId,
                48,
                'Controls the action to perform when clicking the editor gutter with the middle mouse button.'
            )),
            enumDescriptions: [
                ( localizeWithPath(_moduleId, 49, "Add Logpoint.")),
                ( localizeWithPath(_moduleId, 50, "Add Conditional Breakpoint.")),
                ( localizeWithPath(_moduleId, 51, "Add Triggered Breakpoint.")),
                ( localizeWithPath(_moduleId, 52, "Don't perform any action.")),
            ],
            default: 'logpoint',
        },
        'debug.openExplorerOnEnd': {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                53,
                "Automatically open the explorer view at the end of a debug session."
            )),
            default: false
        },
        'debug.closeReadonlyTabsOnEnd': {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                54,
                "At the end of a debug session, all the read-only tabs associated with that session will be closed"
            )),
            default: false
        },
        'debug.inlineValues': {
            type: 'string',
            'enum': ['on', 'off', 'auto'],
            description: ( localizeWithPath(_moduleId, 55, "Show variable values inline in editor while debugging.")),
            'enumDescriptions': [
                ( localizeWithPath(
                    _moduleId,
                    56,
                    "Always show variable values inline in editor while debugging."
                )),
                ( localizeWithPath(
                    _moduleId,
                    57,
                    "Never show variable values inline in editor while debugging."
                )),
                ( localizeWithPath(
                    _moduleId,
                    58,
                    "Show variable values inline in editor while debugging when the language supports inline value locations."
                )),
            ],
            default: 'auto'
        },
        'debug.toolBarLocation': {
            enum: ['floating', 'docked', 'commandCenter', 'hidden'],
            markdownDescription: ( localizeWithPath(
                _moduleId,
                59,
                "Controls the location of the debug toolbar. Either `floating` in all views, `docked` in the debug view, `commandCenter` (requires {0}), or `hidden`.",
                '`#window.commandCenter#`'
            )),
            default: 'floating',
            markdownEnumDescriptions: [
                ( localizeWithPath(_moduleId, 60, "Show debug toolbar in all views.")),
                ( localizeWithPath(_moduleId, 61, "Show debug toolbar only in debug views.")),
                ( localizeWithPath(
                    _moduleId,
                    62,
                    "`(Experimental)` Show debug toolbar in the command center."
                )),
                ( localizeWithPath(_moduleId, 63, "Do not show debug toolbar.")),
            ]
        },
        'debug.showInStatusBar': {
            enum: ['never', 'always', 'onFirstSessionStart'],
            enumDescriptions: [( localizeWithPath(_moduleId, 64, "Never show debug in Status bar")), ( localizeWithPath(_moduleId, 65, "Always show debug in Status bar")), ( localizeWithPath(
                _moduleId,
                66,
                "Show debug in Status bar only after debug was started for the first time"
            ))],
            description: ( localizeWithPath(_moduleId, 67, "Controls when the debug Status bar should be visible.")),
            default: 'onFirstSessionStart'
        },
        'debug.internalConsoleOptions': INTERNAL_CONSOLE_OPTIONS_SCHEMA,
        'debug.console.closeOnEnd': {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                68,
                "Controls if the Debug Console should be automatically closed when the debug session ends."
            )),
            default: false
        },
        'debug.terminal.clearBeforeReusing': {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                69,
                "Before starting a new debug session in an integrated or external terminal, clear the terminal."
            )),
            default: false
        },
        'debug.openDebug': {
            enum: ['neverOpen', 'openOnSessionStart', 'openOnFirstSessionStart', 'openOnDebugBreak'],
            default: 'openOnDebugBreak',
            description: ( localizeWithPath(_moduleId, 70, "Controls when the debug view should open."))
        },
        'debug.showSubSessionsInToolBar': {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                71,
                "Controls whether the debug sub-sessions are shown in the debug tool bar. When this setting is false the stop command on a sub-session will also stop the parent session."
            )),
            default: false
        },
        'debug.console.fontSize': {
            type: 'number',
            description: ( localizeWithPath(_moduleId, 72, "Controls the font size in pixels in the Debug Console.")),
            default: isMacintosh ? 12 : 14,
        },
        'debug.console.fontFamily': {
            type: 'string',
            description: ( localizeWithPath(_moduleId, 73, "Controls the font family in the Debug Console.")),
            default: 'default'
        },
        'debug.console.lineHeight': {
            type: 'number',
            description: ( localizeWithPath(
                _moduleId,
                74,
                "Controls the line height in pixels in the Debug Console. Use 0 to compute the line height from the font size."
            )),
            default: 0
        },
        'debug.console.wordWrap': {
            type: 'boolean',
            description: ( localizeWithPath(_moduleId, 75, "Controls if the lines should wrap in the Debug Console.")),
            default: true
        },
        'debug.console.historySuggestions': {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                76,
                "Controls if the Debug Console should suggest previously typed input."
            )),
            default: true
        },
        'debug.console.collapseIdenticalLines': {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                77,
                "Controls if the Debug Console should collapse identical lines and show a number of occurrences with a badge."
            )),
            default: true
        },
        'debug.console.acceptSuggestionOnEnter': {
            enum: ['off', 'on'],
            description: ( localizeWithPath(
                _moduleId,
                78,
                "Controls whether suggestions should be accepted on Enter in the Debug Console. Enter is also used to evaluate whatever is typed in the Debug Console."
            )),
            default: 'off'
        },
        'launch': {
            type: 'object',
            description: ( localizeWithPath(
                _moduleId,
                79,
                "Global debug launch configuration. Should be used as an alternative to 'launch.json' that is shared across workspaces."
            )),
            default: { configurations: [], compounds: [] },
            $ref: launchSchemaId
        },
        'debug.focusWindowOnBreak': {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                80,
                "Controls whether the workbench window should be focused when the debugger breaks."
            )),
            default: true
        },
        'debug.focusEditorOnBreak': {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                81,
                "Controls whether the editor should be focused when the debugger breaks."
            )),
            default: true
        },
        'debug.onTaskErrors': {
            enum: ['debugAnyway', 'showErrors', 'prompt', 'abort'],
            enumDescriptions: [( localizeWithPath(_moduleId, 82, "Ignore task errors and start debugging.")), ( localizeWithPath(_moduleId, 83, "Show the Problems view and do not start debugging.")), ( localizeWithPath(_moduleId, 84, "Prompt user.")), ( localizeWithPath(_moduleId, 85, "Cancel debugging."))],
            description: ( localizeWithPath(
                _moduleId,
                86,
                "Controls what to do when errors are encountered after running a preLaunchTask."
            )),
            default: 'prompt'
        },
        'debug.showBreakpointsInOverviewRuler': {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                87,
                "Controls whether breakpoints should be shown in the overview ruler."
            )),
            default: false
        },
        'debug.showInlineBreakpointCandidates': {
            type: 'boolean',
            description: ( localizeWithPath(
                _moduleId,
                88,
                "Controls whether inline breakpoints candidate decorations should be shown in the editor while debugging."
            )),
            default: true
        },
        'debug.saveBeforeStart': {
            description: ( localizeWithPath(
                _moduleId,
                89,
                "Controls what editors to save before starting a debug session."
            )),
            enum: ['allEditorsInActiveGroup', 'nonUntitledEditorsInActiveGroup', 'none'],
            enumDescriptions: [
                ( localizeWithPath(
                    _moduleId,
                    90,
                    "Save all editors in the active group before starting a debug session."
                )),
                ( localizeWithPath(
                    _moduleId,
                    91,
                    "Save all editors in the active group except untitled ones before starting a debug session."
                )),
                ( localizeWithPath(_moduleId, 92, "Don't save any editors before starting a debug session.")),
            ],
            default: 'allEditorsInActiveGroup',
            scope: ConfigurationScope.LANGUAGE_OVERRIDABLE
        },
        'debug.confirmOnExit': {
            description: ( localizeWithPath(
                _moduleId,
                93,
                "Controls whether to confirm when the window closes if there are active debug sessions."
            )),
            type: 'string',
            enum: ['never', 'always'],
            enumDescriptions: [
                ( localizeWithPath(_moduleId, 94, "Never confirm.")),
                ( localizeWithPath(_moduleId, 95, "Always confirm if there are debug sessions.")),
            ],
            default: 'never'
        },
        'debug.disassemblyView.showSourceCode': {
            type: 'boolean',
            default: true,
            description: ( localizeWithPath(_moduleId, 96, "Show Source Code in Disassembly View."))
        },
        'debug.autoExpandLazyVariables': {
            type: 'boolean',
            default: false,
            description: ( localizeWithPath(
                _moduleId,
                97,
                "Automatically show values for variables that are lazily resolved by the debugger, such as getters."
            ))
        },
        'debug.enableStatusBarColor': {
            type: 'boolean',
            description: ( localizeWithPath(_moduleId, 98, "Color of the Status bar when debugger is active.")),
            default: true
        },
        'debug.hideLauncherWhileDebugging': {
            type: 'boolean',
            markdownDescription: ( localizeWithPath(
                _moduleId,
                99,
                "Hide 'Start Debugging' control in title bar of 'Run and Debug' view while debugging is active. Only relevant when {0} is not `docked`.",
                '`#debug.toolBarLocation#`'
            )),
            default: false
        }
    }
});
